/*
 * This program gives orders to the stepper card through the
 * serial port.
 * 
 * You may pass in argument the path to your serial device.
 * Default device is /dev/ttyS0
 * 
 * Orders are given on stdin in the form:
 * 
 * "MODE(0-3) cw|ccw <delay> <nsteps>\n"
 *
 * Examples of use:
 * 
 * Turn 200 steps clockwise in one phase mode,
 * one step every 49*0.0001024 s (approx 1 turn/s).
 *         echo "0 cw 49 200" | ./driver
 * 
 * Turn 50 steps counterclockwise in two phases mode,
 * one step every 12*0.0001024 s.
 *         echo "1 ccw 12 50" | ./driver
 * 
 * Turn 200 steps clockwise in half step mode,
 * one step every 6*0.0001024 s.
 *         echo "2 cw 6 200" | ./driver
 */

#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>

#include <string.h>
#include <stdio.h>

static const char *DEFAULT_SERIAL_DEVICE = "/dev/ttyS0";
static const unsigned int DATASIZE = 6;
static const unsigned int T_MAXLEN = 3;
static const char *usage = "Usage: MODE(0-3) cw|ccw <delay> <nsteps>\n";
static const char *format = "%u %3s %u %u\n";
static const char *cw_keyword = "cw";
static const char *ccw_keyword = "ccw";
static const char *err_bad_keyword = "Orientation should be \"cw\" "
                                     "or \"ccw\".\n";
static const char *device_err = "Could not open the specified device.\n";
static struct termios old;

static int open_serial(const char *serial_device)
{
	int fd;
	struct termios tty;

	if ((fd = open(serial_device, O_RDWR)) < 0) {
    	return -1;
	}
	
	tcgetattr(fd, &tty);
	tcgetattr(fd, &old);
	
	memset(&tty, 0, sizeof(tty));
	
	tty.c_cflag = 
		B115200 |
		CS8 |
		CLOCAL |
		CREAD;
	
	tty.c_iflag = IGNPAR;
	tty.c_oflag = 0;
	tty.c_lflag = 0;
	
    tty.c_cc[VTIME] = 1;
	tty.c_cc[VMIN]  = 0;
    
	tcflush(fd, TCIFLUSH); 
	
	tcsetattr(fd, TCSANOW, &tty);
	
	return fd;
}

static void close_serial(int fd)
{
	tcsetattr(fd, TCSAFLUSH, &old);
	close(fd);
}

static void send_command(int fd, unsigned char *data)
{
	data[DATASIZE-1] = 0x00 - (  data[0]
	                           + data[1]
	                           + data[2]
	                           + data[3]
	                           + data[4]);
	write(fd, data, DATASIZE);
}

int main(const int argc, const char **argv)
{
	int serial = open_serial(2 == argc ?
	                                   argv[1] :
	                                   DEFAULT_SERIAL_DEVICE);
	if (0 > serial) {
		fprintf(stderr, device_err);
		return EXIT_FAILURE;
	}
	
	unsigned char data[DATASIZE];
	char str[T_MAXLEN+1];
	
	for (;;) {
		unsigned int delay, nsteps, mode;
		
		int tmp = scanf(format, &mode, str, &delay, &nsteps);
		
		if (EOF == tmp)
			break;
		else if (4 != tmp) {
			fprintf(stderr, usage);
			continue;
		}
		
		if (!strcasecmp(ccw_keyword, str))
			data[0] = 0x00;
		else if (!strcasecmp(cw_keyword, str))
			data[0] = 0x04;
		else {
			fprintf(stderr, err_bad_keyword);
			continue;
		}
		
		mode %= 4;
		data[0] |= (unsigned char) mode;
		data[1] = delay / 256;
		data[2] = delay % 256;
		data[3] = nsteps / 256;
		data[4] = nsteps % 256;
		
		send_command(serial, data);
	}
	
	close_serial(serial);
	
	return EXIT_SUCCESS;
}

